//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library, Example3 - Multiple Devices
//
// Date: 7th January, 2004
// Last Updated: 18th May 2013
// Author: Darren John Dwyer
// Copyright: (c) 2002-2013 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example calls enumerates all devices using the TDx_Draw::DDEnumerate()
// method and the TDx_Draw::OnDDEnumerate() event.
//
// During each device enumeration it creates a TDx_Draw component using
// runtime code and stores the component into a global TList, a technique that
// can be utilized to create multiple directdraw devices.
//
// After enumeration it displays the capabilities of each device before finally
// destroying all devices.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
TDx_Draw_Component_List = new TList();
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
delete TDx_Draw_Component_List;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
DevicesListBox->MultiSelect=true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
ClearDevicesButtonClick(this);
DestroyActiveButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::EnumerateButtonClick(TObject *Sender)
{
Msg("Scanning for Available Devices");
NumDevices = 0;
DevicesListBox->Items->Clear();
Dx_Draw1->DDEnumerate(NULL,0);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SelectAllDevicesButtonClick(TObject *Sender)
{
Msg("Selecting All Available Devices");
for (int i=0;i<DevicesListBox->Items->Count;i++)
    {
    DevicesListBox->Selected[i] = true;
    DevicesListBox->OnClick(this);
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (DevicesListBox->ItemIndex>=0)
    {
    AnsiString name = Names[DevicesListBox->ItemIndex];
    AnsiString desc = Descriptions[DevicesListBox->ItemIndex];

    Msg("Device Selected: '"+name+", "+desc+"'");
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateSelectedButtonClick(TObject *Sender)
{
Msg("Creating all Selected Devices");
for (int i=0;i<DevicesListBox->Items->Count;i++)
    {
    TDx_Draw* temp_tdx_draw = (TDx_Draw*) TDx_Draw_Component_List->Items[i];
    if (temp_tdx_draw!=NULL)
        if (DevicesListBox->Selected[i]) temp_tdx_draw->Create( Guids[i] );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RetrieveCapsButtonClick(TObject *Sender)
{
HelCapsListBox->Items->Clear();
DriverCapsListBox->Items->Clear();

Msg("Retrieving Device Capabilities");
for (int i=0;i<DevicesListBox->Items->Count;i++)
    if (DevicesListBox->Selected[i])
        {
        TDx_Draw* temp_tdx_draw = (TDx_Draw*) TDx_Draw_Component_List->Items[i];
        if (temp_tdx_draw!=NULL)
            if (temp_tdx_draw->GetCaps( DriverCaps, HelCaps ))
                {
                // insert caps directly into listboxes
                HelCapsListBox->Items->Add( "------ "+Names[i]+", "+Descriptions[i]+" ------" );
                HelCapsListBox->Items->AddStrings( HelCaps->Caps_Strings );
                DriverCapsListBox->Items->Add( "------ "+Names[i]+", "+Descriptions[i]+" ------" );
                DriverCapsListBox->Items->AddStrings( DriverCaps->Caps_Strings );

                // check for particular caps by comparing the 'public:' property using DirectX bitwise flags
                if (DriverCaps->Caps & DDCAPS_3D) Msg("Note: 3D Hardware Acceleration Available");
                if (DriverCaps->Caps & DDCAPS_BANKSWITCHED) Msg("Note: Display is Bank Switched Hardware: Relatively Slow Video Access");
                if (HelCaps->FXCaps & DDFXCAPS_BLTROTATION) Msg("Note: Hardware Emulation Layer supports Hardware Blit Rotation of arbitrary angle." );

                // [OR]
                // check for particular caps by comparing the '__published' property using the vcl's TSet<> operators
                if (DriverCaps->Caps_.Contains(ddcaps_bankswitched)) Msg("Note: Display is Bank Switched Hardware: Relatively Slow Video Access");
                if (DriverCaps->Caps_.Contains(ddcaps_blt)) Msg("Note: Hardware supports Bit-Blitting functions");
                if (HelCaps->FXCaps_.Contains(ddfxcaps_bltrotation90)) Msg("Note: Hardware Emulation Layer supports Hardware Blit Rotation of 90 degrees." );
                }
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyActiveButtonClick(TObject *Sender)
{
Msg("Destroying Currently Active Device");
for (int i=0;i<DevicesListBox->Items->Count;i++)
    {
    TDx_Draw* temp_tdx_draw = (TDx_Draw*) TDx_Draw_Component_List->Items[i];
    if (temp_tdx_draw!=NULL)
        if (temp_tdx_draw->Created) temp_tdx_draw->Destroy();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
DestroyActiveButtonClick(this);
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearDevicesButtonClick(TObject *Sender)
{
if (DevicesListBox->Items->Count>0)
    {
    Msg( "Clearing Available Devices List" );
    DevicesListBox->ItemIndex = -1;
    DevicesListBox->Items->Clear();
    DevicesListBox->OnClick(this);
    }
else
    {
    DevicesListBox->ItemIndex = -1;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
AnsiString name = Names[DevicesListBox->ItemIndex];
AnsiString desc = Descriptions[DevicesListBox->ItemIndex];

Msg("Device Created: '"+name+", "+desc+"'");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
AnsiString name = Names[DevicesListBox->ItemIndex];
AnsiString desc = Descriptions[DevicesListBox->ItemIndex];

Msg("Device Destroyed: '"+name+", "+desc+"'");

HelCapsListBox->Items->Text = "------ none ------";
DriverCapsListBox->Items->Text = "------ none ------";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1DDEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverName, void *Context,
    HMONITOR HMonitor, bool &Finished)
{
if (NumDevices<MAX_DEVICES)
    {
    Guids[NumDevices] = Guid;
    Names[NumDevices] = DriverName;
    Descriptions[NumDevices] = DriverDescription;
    NumDevices++;

    Msg("Device Found: '"+DriverName+", "+DriverDescription+"'");
    DevicesListBox->Items->Add( DriverName+", "+DriverDescription );

    // create a temporary TDx_Draw component for each device
    TDx_Draw* temp_tdx_draw = new TDx_Draw(this);

    // set the events for this component to use the existing IDE-visible
    // events of the Dx_Draw1 component
    if (temp_tdx_draw!=NULL)
        {
        temp_tdx_draw->OnError = Dx_Draw1->OnError;
        temp_tdx_draw->OnCreate = Dx_Draw1->OnCreate;
        temp_tdx_draw->OnDestroy = Dx_Draw1->OnDestroy;
        }

    // add it to a TList for easy access later.
    // note: add NULL's to keep list sychronized with Guids[], etc. arrays
    TDx_Draw_Component_List->Add( (void*) temp_tdx_draw );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------



